@@ -27,9 +27,9 @@
 #include <config.h>
 #include <string.h>		/* For memset() */
 
-#include "gdkalias.h"
 #include "gdk.h"
 #include "gdkinternals.h"
+#include "gdkalias.h"
 
 typedef struct _GdkIOClosure GdkIOClosure;
 
@@ -255,7 +255,6 @@ gdk_event_put (GdkEvent *event)
   gdk_display_put_event (display, event);
 }
 
-static GMemChunk *event_chunk = NULL;
 static GHashTable *event_hash = NULL;
 
 /**
@@ -275,17 +274,10 @@ gdk_event_new (GdkEventType type)
   GdkEventPrivate *new_private;
   GdkEvent *new_event;
   
-  if (event_chunk == NULL)
-    {
-      event_chunk = g_mem_chunk_new ("events",
-				     sizeof (GdkEventPrivate),
-				     4096,
-				     G_ALLOC_AND_FREE);
-      event_hash = g_hash_table_new (g_direct_hash, NULL);
-    }
-  
-  new_private = g_chunk_new (GdkEventPrivate, event_chunk);
-  memset (new_private, 0, sizeof (GdkEventPrivate));
+  if (!event_hash)
+    event_hash = g_hash_table_new (g_direct_hash, NULL);
+
+  new_private = g_slice_new0 (GdkEventPrivate);
   
   new_private->flags = 0;
   new_private->screen = NULL;
@@ -446,8 +438,6 @@ gdk_event_free (GdkEvent *event)
 {
   g_return_if_fail (event != NULL);
 
-  g_assert (event_chunk != NULL); /* paranoid */
-  
   if (event->any.window)
     g_object_unref (event->any.window);
   
@@ -475,8 +465,7 @@ gdk_event_free (GdkEvent *event)
 
     case GDK_BUTTON_PRESS:
     case GDK_BUTTON_RELEASE:
-      if (event->button.axes)
-	g_free (event->button.axes);
+      g_free (event->button.axes);
       break;
       
     case GDK_EXPOSE:
@@ -485,8 +474,7 @@ gdk_event_free (GdkEvent *event)
       break;
       
     case GDK_MOTION_NOTIFY:
-      if (event->motion.axes)
-	g_free (event->motion.axes);
+      g_free (event->motion.axes);
       break;
       
     case GDK_SETTING:
@@ -498,7 +486,7 @@ gdk_event_free (GdkEvent *event)
     }
 
   g_hash_table_remove (event_hash, event);
-  g_mem_chunk_free (event_chunk, event);
+  g_slice_free (GdkEventPrivate, (GdkEventPrivate*) event);
 }
 
 /**
@@ -561,6 +549,7 @@ gdk_event_get_time (GdkEvent *event)
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
       case GDK_OWNER_CHANGE:
+      case GDK_GRAB_BROKEN:
         /* return current time */
         break;
       }
@@ -637,6 +626,7 @@ gdk_event_get_state (GdkEvent        *event,
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
       case GDK_OWNER_CHANGE:
+      case GDK_GRAB_BROKEN:
         /* no state field */
         break;
       }
@@ -842,6 +832,36 @@ gdk_event_get_axis (GdkEvent   *event,
   return gdk_device_get_axis (device, axes, axis_use, value);
 }
 
+/**
+ * gdk_event_request_motions:
+ * @event: a valid #GdkEvent
+ *
+ * Request more motion notifies if @event is a motion notify hint event.
+ * This function should be used instead of gdk_window_get_pointer() to
+ * request further motion notifies, because it also works for extension
+ * events where motion notifies are provided for devices other than the
+ * core pointer. Coordinate extraction, processing and requesting more
+ * motion events from a %GDK_MOTION_NOTIFY event usually works like this:
+ *
+ * <informalexample><programlisting>
+ * {  // motion_event handler
+ *   x = motion_event->x;
+ *   y = motion_event->y;
+ *   ; // handle (x,y) motion
+ *   gdk_event_request_motions (motion_event); // handles is_hint events
+ * }
+ * </programlisting></informalexample>
+ *
+ * Since: 2.12
+ **/
+void
+gdk_event_request_motions (GdkEventMotion *event)
+{
+  g_return_if_fail (event != NULL);
+  if (event->type == GDK_MOTION_NOTIFY && event->is_hint)
+    gdk_device_get_state (event->device, event->window, NULL, NULL);
+}
+
 /**
  * gdk_event_set_screen:
  * @event: a #GdkEvent
@@ -873,7 +893,7 @@ gdk_event_set_screen (GdkEvent  *event,
  * Returns the screen for the event. The screen is
  * typically the screen for <literal>event->any.window</literal>, but
  * for events such as mouse events, it is the screen
- * where the the pointer was when the event occurs -
+ * where the pointer was when the event occurs -
  * that is, the screen which has the root window 
  * to which <literal>event->motion.x_root</literal> and
  * <literal>event->motion.y_root</literal> are relative.
@@ -1198,7 +1218,7 @@ gdk_event_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("GdkEvent",
+    our_type = g_boxed_type_register_static (g_intern_static_string ("GdkEvent"),
 					     (GBoxedCopyFunc)gdk_event_copy,
 					     (GBoxedFreeFunc)gdk_event_free);
   return our_type;
@@ -1221,3 +1241,6 @@ gdk_setting_get (const gchar *name,
 {
   return gdk_screen_get_setting (gdk_screen_get_default (), name, value);
 }
+
+#define __GDK_EVENTS_C__
+#include "gdkaliasdef.c"
